#!/usr/bin/env python3

from utils.sympy import command_law_solve


def get_acc_cmd_1(s, vals):
    """
    Constant: acc = k1 -> 3 unknowns

    t       0   f
    acc     -   n
    vel     Y   n
    pos     Y   Y
    """
    print('1 - constant')

    acc_cmd1 = command_law_solve(
        symbs=s,
        acc=s['k1'],
        af=s['k2'], vf=s['k3'],
    )
    print('\tacc_cmd1', acc_cmd1)
    # acc_cmd = 2*(-p0 + pf - tgo*v0)/tgo**2
    #         = 2/tgo*(-v0 + 1/tgo*(pf-p0))

    return acc_cmd1


def get_acc_cmd_2(s, vals):
    """
    Constant b: acc = k1 -> 3 unknowns

            0   f
    acc     -   n
    vel     n   Y
    pos     Y   Y

    # ====>>>>>> NOPE NOPE
    """
    print('2 - constant // NOPE')

    acc_cmd2 = command_law_solve(
        symbs=s,
        acc=s['k1'],
        v0=s['k2'], af=s['k3'],
    )
    print('\tacc_cmd2', acc_cmd2)
    # acc_cmd = 2*(p0 - pf + tgo*vf)/tgo**2
    #         = 2/tgo * (vf - 1/tgo*(pf-p0))

    return acc_cmd2


def get_acc_cmd_3(s, vals):
    """
    Linear a: acc = k1*t + k2 -> 4 unknowns

            0   f
    acc     -   n
    vel     Y   y
    pos     Y   Y
    """
    print('3 - linear')

    acc_cmd3 = command_law_solve(
        symbs=s,
        acc=s['k1']*s['t']+s['k2'],
        af=s['k3'],
    )
    print('\tacc_cmd3', acc_cmd3)
    # acc_cmd = 2*(-3*p0 + 3*pf - 2*tgo*v0 - tgo*vf)/tgo**2
    #         = 2/tgo * (-(vf+2*v0) + 3/tgo*(pf-p0))

    return acc_cmd3


def get_acc_cmd_4(s, vals):
    """
    Linear b: acc = k1*t + k2 -> 4 unknowns

            0   f
    acc     -   y
    vel     n   y
    pos     Y   Y

    # ====>>>>>> NOPE NOPE
    """
    print('4 - linear // NOPE')

    acc_cmd4 = command_law_solve(
        symbs=s,
        acc=s['k1']*s['t']+s['k2'],
        v0=s['k3'],
    )
    print('\tacc_cmd4', acc_cmd4)
    # acc_cmd = 2*(-af*tgo**2 + 3*p0 - 3*pf + 3*tgo*vf)/tgo**2
    #         = -2*af + 2/tgo * (3*vf - 3/tgo*(pf-p0))

    return acc_cmd4


def get_acc_cmd_5(s, vals):
    """
    Quadratic: acc = k1*t**2 + k2*t + k3 -> 5 unknowns

            0   f
    acc     -   y
    vel     Y   y
    pos     Y   Y
    """
    print('5 - quadratic')

    acc_cmd5 = command_law_solve(
        symbs=s,
        acc=s['k1']*s['t']**2+s['k2']*s['t']+s['k3'],
    )
    print('\tacc_cmd5', acc_cmd5)
    # acc_cmd = (af*tgo**2 - 12*p0 + 12*pf - 6*tgo*v0 - 6*tgo*vf)/tgo**2
    #         = af - 6/tgo*(vf+v0) + 12/tgo**2*(pf-p0)

    return acc_cmd5


def get_acc_cmd_6(s, vals):
    """
    Cubic: acc = k1*t**3 + k2*t**2 + k3*t + k4 -> 6 unknowns (5 known + 1 free param)

            0   f
    acc     -   y
    vel     Y   y
    pos     Y   Y
    """
    print('6 - cubic')

    acc_cmd6 = command_law_solve(
        symbs=s,
        acc=s['k1']*s['t']**3+s['k2']*s['t']**2+s['k3']*s['t']+s['k4'],
        constraintJerk=True,
    )
    print('\tacc_cmd6', acc_cmd6)
    # acc_cmd = (3*af*tgo**2 - 20*p0 + 20*pf - 4*tgo*(2*v0 + 3*vf))/tgo**2

    return acc_cmd6
